package com.perfectech.tis;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.concurrent.atomic.AtomicInteger;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.*;
import android.database.sqlite.SQLiteDatabase.CursorFactory;
import android.util.Log;

public class TISDBHelper extends SQLiteOpenHelper {
	// Declaring Local Variables for database name , and database path and
	// sqlite object and my context
	private static String DB_PATH = "";
	private static final String DB_NAME = "TIS.db"; // i had created this in my
													// assets folder
	private SQLiteDatabase myDataBase;
	private final Context myContext;
	private static TISDBHelper mDBConnection;
	static int version_val = 5; // version of my table
	private AtomicInteger mOpenCounter = new AtomicInteger();

	// Constructor which sends db name to SQLiteOpenHelper
	public TISDBHelper(Context context, String name, CursorFactory factory,
			int version) {
		super(context, name, factory, version);
		this.myContext = context;
		DB_PATH = "/data/data/"
				+ context.getApplicationContext().getPackageName()
				+ "/databases/";
		Log.d("DB_PATH: ", DB_PATH);
		Log.d("DB_NAME: ", name);
	}

	// Used to get the Object

	public static synchronized TISDBHelper getDBAdapterInstance(Context context) {
		if (mDBConnection == null) {
			mDBConnection = new TISDBHelper(context, DB_NAME, null, version_val);
		}
		return mDBConnection;
	}

	@Override
	public void onCreate(SQLiteDatabase db) {
		// TODO Auto-generated method stub

	}

	@Override
	public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
		// TODO Auto-generated method stub
		if (oldVersion < 3) {
            try {
                db.execSQL("ALTER TABLE jobs ADD COLUMN pickup_signature_name TEXT");
                db.execSQL("ALTER TABLE jobs ADD COLUMN delivery_signature_name TEXT");
            } catch (Exception ex) {

            }
		}
		if (oldVersion < 4) {
            try {
                db.execSQL("CREATE TABLE gpsHistory (_id INTEGER PRIMARY KEY, userID TEXT, JobID TEXT, Lat TEXT, Long TEXT, Time TEXT, Accuracy TEXT, Speed TEXT, Distance Text, Type Text)");
            } catch (Exception ex ){

            }
		}
		if (oldVersion < 5) {
            try {
                db.execSQL("CREATE TABLE DVIRFieldGroups (_id INTEGER PRIMARY KEY, Name TEXT, SortOrder INTEGER)");
                db.execSQL("ALTER TABLE DVIRAssignedFields ADD COLUMN FieldGroupID INTEGER");
                db.execSQL("ALTER TABLE DVIRAssignedFields ADD COLUMN SortOrder INTEGER");
            } catch (Exception ex) {

            }
		}
	}

	public void createDataBase() throws IOException {
		boolean dbExist = checkDataBase();
		if (dbExist) {
			// do nothing - database already exist
		} else {

			// By calling following method
			// 1) an empty database will be created into the default system path
			// of your application
			// 2) than we overwrite that database with our database.
			this.getReadableDatabase();
			try {
				copyDataBase();
			} catch (IOException e) {
				throw new Error("Error copying database");
			}
		}
	}

	private boolean checkDataBase() {
		SQLiteDatabase checkDB = null;
		try {
			String myPath = DB_PATH + DB_NAME;
			Log.d("MyPath", myPath);
			checkDB = SQLiteDatabase.openDatabase(myPath, null,
					SQLiteDatabase.OPEN_READONLY);

		} catch (SQLiteException e) {
			// database does't exist yet.
		}
		if (checkDB != null) {
			checkDB.close();
		}

		return checkDB != null ? true : false;
	}

	private void copyDataBase() throws IOException {
		// Open your local db as the input stream
		InputStream myInput = myContext.getAssets().open(DB_NAME);

		// Path to the just created empty db
		String outFileName = DB_PATH + DB_NAME;

		// Open the empty db as the output stream
		OutputStream myOutput = new FileOutputStream(outFileName);

		// transfer bytes from the inputfile to the outputfile
		byte[] buffer = new byte[1024];
		int length;
		while ((length = myInput.read(buffer)) > 0) {
			myOutput.write(buffer, 0, length);
		}

		// Close the streams
		myOutput.flush();
		myOutput.close();
		myInput.close();

		String myPath = DB_PATH + DB_NAME;
		myDataBase = SQLiteDatabase.openDatabase(myPath, null,
				SQLiteDatabase.OPEN_READWRITE);
		myDataBase = this.getWritableDatabase();
		myDataBase.setVersion(version_val);
	}

	public void openDataBase() throws SQLException {
		//We only need to open the database connection if it is not already open
		if(mOpenCounter.incrementAndGet() == 1) {
			String myPath = DB_PATH + DB_NAME;
			myDataBase = SQLiteDatabase.openDatabase(myPath, null,
					SQLiteDatabase.OPEN_READWRITE);
			myDataBase = this.getWritableDatabase();
		}
	}

	@Override
	public synchronized void close() {
		//Only close the database if there is only one open connection
		if (mOpenCounter.decrementAndGet() == 0) {
			if (myDataBase != null)
				myDataBase.close();
			super.close();
		}
	}

	public ArrayList<ArrayList<String>> selectRecordsFromDBList(String query,
			String[] selectionArgs) {
		ArrayList<ArrayList<String>> retList = new ArrayList<ArrayList<String>>();
		ArrayList<String> list = new ArrayList<String>();
		Cursor cursor = myDataBase.rawQuery(query, selectionArgs);
		if (cursor.moveToFirst()) {
			do {
				list = new ArrayList<String>();
				for (int i = 0; i < cursor.getColumnCount(); i++) {
					//With isNull here a null filed in database will show as a blank in string list
					if (cursor.isNull(i)) {
						list.add("");
					} else {
						list.add(cursor.getString(i));
					}
				}
				retList.add(list);
			} while (cursor.moveToNext());
		}
		if (cursor != null && !cursor.isClosed()) {
			cursor.close();
		}
		return retList;
	}

	public ArrayList<String> getList(String query, String[] selectionArgs) {

		ArrayList<String> list = new ArrayList<String>();

		// calls the DBAdapter for the cursor (again, best way I can describe
		// it)
		Cursor results = myDataBase.rawQuery(query, selectionArgs);

		// loop to get all rows
		if (results != null) {
			if (results.moveToFirst()) {
				do {
					String[] names = results.getColumnNames();
					int length = names.length;
					for (int i = length - 1; i >= 0; i--) {
						//With isNull here a null filed in database will show as a blank in string list
						if (results.isNull(i)) {
							list.add("");
						} else {
							list.add(results.getString(i));
						}
					}
				} while (results.moveToNext()); // move to next row
			} // if
		} // if

		// adds the results to the array
		return list;
	}

	public long insertRecordsInDB(String tableName, String nullColumnHack,
			ContentValues initialValues) {
		return myDataBase.insert(tableName, nullColumnHack, initialValues);
	}

	public long updateRecordsInDB(String tableName, String sWhereClause, ContentValues initialValues) {
		return myDataBase.update(tableName, initialValues, sWhereClause, null);		
	}
	
	public int deleteRecordInDB(String tableName, String whereClause) {
		return myDataBase.delete(tableName, whereClause, null);
	}
	
	public int deleteAllRecords(String tableName) {
		return myDataBase.delete(tableName, null, null);
	}

}
