package com.perfectech.tis.activities;

import android.content.ContentValues;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.TextView;
import android.widget.EditText;
import android.widget.Spinner;
import android.view.ViewGroup;
import android.graphics.Color;
import android.widget.BaseAdapter;
import android.widget.SpinnerAdapter;
import android.widget.AdapterView;

import com.perfectech.tis.DBLogs;
import com.perfectech.tis.DateConvertClass;
import com.perfectech.tis.objects.MsgUser;
import com.perfectech.tis.R;
import com.perfectech.tis.syncClasses.SyncTasks;
import com.perfectech.tis.TISDBHelper;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class CreateCommActivity extends AppCompatActivity {

    private static final String sDBName = "TIS.db";
    private static final int iVersionVal = 5;
    private String sUserID = "";
    private String sInMessageText = "";
    private String sInFromUserName = "";
    private String sInFromUserFullName = "";
    private String sToUserName = "";
    private String sToFullUserName = "";
    String sLoggedInUserCompany = "";
    private static ArrayList<ArrayList<String>> userArray;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_create_comm);

        SharedPreferences prefs = getSharedPreferences("TISSharedPrefs",
                Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = prefs.edit();
        sLoggedInUserCompany = prefs.getString("LoggedInUserCompany", "No Value");
        sUserID = prefs.getString("UserID", "No Value");
        TextView otvUserName = (TextView) findViewById(R.id.tvUserName);
        otvUserName.setText(sLoggedInUserCompany.substring(0,sLoggedInUserCompany.indexOf('-')));

        // see if a message was pass in
        sInMessageText = getIntent().getExtras().getString("InMessageText");
        sInFromUserName = getIntent().getExtras().getString("InFromUserName");
        sInFromUserFullName = getIntent().getExtras().getString("InFromUserFullName");

    }

    @Override
    public void onResume() {
        super.onResume();

        // if there is a message passed in the this is a reply must added it to the window
        if (this.sInMessageText.length() > 0) {
            String sReplayMsg = sInMessageText;
            EditText etMessage = (EditText) findViewById(R.id.etMessage);
            etMessage.setText(sReplayMsg);

            Spinner spUsers = (Spinner) findViewById(R.id.spUsers);
            //spUsers.setOnItemSelectedListener(this);

            ArrayList<MsgUser> msgUser = new ArrayList<MsgUser>();
            msgUser.add(new MsgUser(sInFromUserFullName, sInFromUserName));
            MyAdapter adapter = new MyAdapter(msgUser);
            spUsers.setAdapter(adapter);

            sToFullUserName = sInFromUserFullName;
            sToUserName = sInFromUserName;

            spUsers.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
                /**
                 * Called when a new item was selected (in the Spinner)
                 */
                public void onItemSelected(AdapterView<?> parent,
                                           View view, int pos, long id) {
                    MsgUser g = (MsgUser) parent.getItemAtPosition(pos);
                }

                public void onNothingSelected(AdapterView parent) {
                    // Do nothing.
                }
            });

        } else {
            // get a valid list of people that the person can send a message to
            getUserList();
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_create_comm, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_send) {
            sendMessage();
            finish();
            return true;
        }
        if (id == R.id.action_cancel) {
            finish();
            return true;
        }
        if (id == R.id.action_icon_meanings) {
            Intent i = new Intent(getApplicationContext(), HelpActivity.class);
            i.putExtra("ScreenID", "IconMeanings");
            startActivity(i);
            return true;
        }
        if (id == R.id.action_help) {
            Intent i = new Intent(getApplicationContext(), HelpActivity.class);
            i.putExtra("ScreenID", "CreateCommActivity");
            startActivity(i);
            return true;
        }
        if (id == R.id.action_about) {
            Intent i = new Intent(getApplicationContext(), AboutActivity.class);
            startActivity(i);
            return true;
        }

        return super.onOptionsItemSelected(item);
    }

    private void getUserList() {
        Spinner spUsers = (Spinner) findViewById(R.id.spUsers);

        ArrayList<MsgUser> msgUser = new ArrayList<MsgUser>();

        // query the EmployeeMaster table from the database
        TISDBHelper oDB = new TISDBHelper(getApplicationContext(), sDBName,
                null, iVersionVal);
        try {
            oDB.createDataBase();
        } catch (IOException ioe) {
            throw new Error("Unable to create database");
        }

        oDB.openDataBase();
        String sSQL = "Select FirstName||' '||LastName, UserID ";
        sSQL += "from EmployeeMaster ";
        sSQL += " where lower(UserID) <> '" + sUserID + "'";

        Log.d("sSQL", sSQL);

        userArray = oDB.selectRecordsFromDBList(sSQL, null);

        // closes the database
        try {
            oDB.close();
        } catch (Throwable e) {
            e.printStackTrace();
        }

        if (userArray.size() > 0) {
            for (ArrayList<String> sValue : userArray) {
                msgUser.add(new MsgUser(sValue.get(0), sValue.get(1)));
            }
        }

        MyAdapter adapter = new MyAdapter(msgUser);
        spUsers.setAdapter(adapter);

        spUsers.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            /**
             * Called when a new item was selected (in the Spinner)
             */
            public void onItemSelected(AdapterView<?> parent,
                                       View view, int pos, long id) {
                MsgUser g = (MsgUser) parent.getItemAtPosition(pos);
                sToUserName = g.getUserName();
                sToFullUserName = g.getUserFullName();
            }

            public void onNothingSelected(AdapterView parent) {
                // Do nothing.
            }
        });
    }

    private void sendMessage() {
        // get the text of the message
        EditText etMessage = (EditText) findViewById(R.id.etMessage);
        String sMessage = etMessage.getText().toString();

        String sFromFullUserName = sLoggedInUserCompany.substring(0,sLoggedInUserCompany.indexOf('-')-1);

        // write the message to the Messages table
        DateConvertClass oDateConvertClass = new DateConvertClass();
        oDateConvertClass.oContext = getApplicationContext();
        String sCurrentDateTime = oDateConvertClass.GMTTimestamp();

        ContentValues oValues = new ContentValues();
        oValues.put("ToUserFullName", sToFullUserName);
        oValues.put("ToUserName", sToUserName);
        oValues.put("MsgDate", sCurrentDateTime);
        oValues.put("Message", sMessage.replace("\n",""));
        oValues.put("MsgStatus","New");
        oValues.put("FromUserName", sUserID);
        oValues.put("FromUserFullName",sFromFullUserName);

        // insert the values to the table
        TISDBHelper oDB = new TISDBHelper(getApplicationContext(), sDBName, null, iVersionVal);
        try {
            oDB.createDataBase();
        } catch (IOException ioe) {
            throw new Error("Unable to create database");
        }

        try {
            oDB.openDataBase();

            Log.d("insert", "Inserting Messages");
            oDB.insertRecordsInDB("Messages", null, oValues);
        } catch (Exception sqlEx) {
            DBLogs.insert("SQL exception:" + sqlEx.toString(),
                    "CreateCommActivity.sendMessage",
                    getApplicationContext(), sDBName, iVersionVal);
        } finally {
            oDB.close();
        }

        // need to get the id from the newly inserted row
        // query the EmployeeMaster table from the database

        oDB.openDataBase();
        String sSQL = "Select _id ";
        sSQL += "from Messages ";
        sSQL += " where MsgDate = '" + sCurrentDateTime + "'";
        sSQL += " and Message = '" + sMessage.replace("\n","") + "' ";
        sSQL += " and MsgStatus = 'New' ";
        sSQL += " and ToUserName = '" + sToUserName + "'";

        Log.d("sSQL", sSQL);

        userArray = oDB.selectRecordsFromDBList(sSQL, null);

        // closes the database
        try {
            oDB.close();
        } catch (Throwable e) {
            e.printStackTrace();
        }

        String sID = "";
        if (userArray.size() > 0) {
            for (ArrayList<String> sValue : userArray) {
                sID =  sValue.get(0);
            }
        }

        // queue the message to be sent
        SyncTasks syncTasks = new SyncTasks();
        syncTasks.oContext = getApplicationContext();
        syncTasks.sRecType = "CN";
        syncTasks.sRecID = sID;
        syncTasks.sDateTime = sCurrentDateTime;
        syncTasks.sQueued = "0";
        syncTasks.sDateTimeQueued = "";
        syncTasks.sCompleted = "0";
        syncTasks.sDateTimeCompleted = "";
        syncTasks.write();

    }


    private class MyAdapter extends BaseAdapter implements SpinnerAdapter {

        /**
         * The internal data (the ArrayList with the Objects).
         */
        private final List<MsgUser> data;

        public MyAdapter(List<MsgUser> data) {
            this.data = data;
        }

        /**
         * Returns the Size of the ArrayList
         */
        @Override
        public int getCount() {
            return data.size();
        }

        /**
         * Returns one Element of the ArrayList
         * at the specified position.
         */
        @Override
        public Object getItem(int position) {
            return data.get(position);
        }

        @Override
        public long getItemId(int i) {
            return i;
        }

        /**
         * Returns the View that is shown when a element was
         * selected.
         */
        @Override
        public View getView(int position, View recycle, ViewGroup parent) {
            TextView text;
            if (recycle != null) {
                // Re-use the recycled view here!
                text = (TextView) recycle;
            } else {
                // No recycled view, inflate the "original" from the platform:
                text = (TextView) getLayoutInflater().inflate(
                        android.R.layout.simple_dropdown_item_1line, parent, false
                );
            }
            text.setTextColor(Color.BLACK);
            text.setText(data.get(position).UserFullName);
            return text;
        }
    }

}
