package com.perfectech.tis.activities;

import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.TextView;

import com.perfectech.tis.R;
import com.perfectech.tis.objects.DVIR;
import com.perfectech.tis.DVIRReportsTextViewListAdapter;
import com.perfectech.tis.DateConvertClass;
import com.perfectech.tis.TISDBHelper;

import java.io.IOException;
import java.util.ArrayList;

public class DVIRReportsActivity extends AppCompatActivity {
    private static final String sDBName = "TIS.db";
    private static final int iVersionVal = 5;
    private static String sUserID = "";
    private static ArrayList<ArrayList<String>> reportsArray;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_dvirreports);

        SharedPreferences prefs = getSharedPreferences("TISSharedPrefs",
                Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = prefs.edit();
        String sLoggedInUserCompany = prefs.getString("LoggedInUserCompany", "No Value");
        TextView Header = (TextView) findViewById(R.id.tvDVIRReportHdrUserName);
        Header.setText(sLoggedInUserCompany.substring(0,sLoggedInUserCompany.indexOf('-')));
        String sCurrentVersion = prefs.getString("TISVersion", "No Value");
        TextView otvCopyright = (TextView) findViewById(R.id.tvCopyright);
        String sVersionText = otvCopyright.getText().toString();
        // need to replace the %RELNUM% with the actual rel version
        sVersionText = sVersionText.replace("RELNUM", sCurrentVersion);
        otvCopyright.setText(sVersionText);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_dvirreports, menu);
        getReportsFromDB();
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_icon_meanings) {
            Intent i = new Intent(getApplicationContext(), HelpActivity.class);
            i.putExtra("ScreenID", "IconMeanings");
            startActivity(i);
            return true;
        }
        if (id == R.id.action_help) {
            Intent i = new Intent(getApplicationContext(), HelpActivity.class);
            i.putExtra("ScreenID", "DVIRReportsActivity");
            startActivity(i);
            return true;
        }
        if (id == R.id.action_about) {
            Intent i = new Intent(getApplicationContext(), AboutActivity.class);
            startActivity(i);
            return true;
        }

        return super.onOptionsItemSelected(item);
    }

    private void getReportsFromDB () {
        TISDBHelper oDB = new TISDBHelper(getApplicationContext(), sDBName,
                null, iVersionVal);
        try {
            oDB.createDataBase();
        } catch (IOException ioe) {
            throw new Error("Unable to create database");
        }

        oDB.openDataBase();

        // Show the last one entered at the top.
        String sSQL = "select * from DVIREntered order by _id desc";
        Log.d("sSQL", sSQL);

        reportsArray = oDB.selectRecordsFromDBList(sSQL, null);

        // closes the database
        try {
            oDB.close();
        } catch (Throwable e) {
            e.printStackTrace();
        }

        ListView reports = (ListView) this.findViewById(R.id.lvDVIRItemsList);

        DateConvertClass oDateConvertClass = new DateConvertClass();
        oDateConvertClass.oContext = getApplicationContext();
        ArrayList<DVIR> allDVIRReports = new ArrayList<DVIR>();
        if (reportsArray.size() > 0) {
            for (ArrayList<String> sValue : reportsArray) {
                DVIR DVIRObj = new DVIR();
                DVIRObj.CompletedDVIR = sValue.get(0);
                DVIRObj.Remarks = sValue.get(1);
                DVIRObj.OutOfStateMileage = sValue.get(2);
                DVIRObj.Location = sValue.get(3);
                DVIRObj.EndMileage = sValue.get(4);
                DVIRObj.BeginMileage = sValue.get(5);
                DVIRObj.DriverNo = sValue.get(7);
                DVIRObj.TruckNo = sValue.get(8);
                DVIRObj.TrailerNo = sValue.get(9);
                //Convert database GMT timestamp to local device time
                DVIRObj.DVIRDateTime = oDateConvertClass.GMTToLocal(sValue.get(10));
                DVIRObj.DVIRNo = sValue.get(11);
                allDVIRReports.add(DVIRObj);
            }
            reports.setAdapter(new DVIRReportsTextViewListAdapter(this, R.layout.dvir_reports_row, allDVIRReports));
            reports.setOnItemClickListener(new ListView.OnItemClickListener() {
                public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                    try {
                        // get the selected row's DVIR number
                        Integer iCounter = 0;
                        for (ArrayList<String> sValue : reportsArray) {
                            if (position == iCounter) {
                                if (sValue.get(0).toUpperCase().equals("PART")) {
                                    //Initial DVIR was only partially completed so go back and complete
                                    completeReport();
                                } else {
                                    getReport(sValue.get(11));
                                }
                            }
                            iCounter++;
                        }
                    } catch (Exception e) {
                        Log.d("Error:", e.toString());
                    }
                }
            });

        } else {
            // No Reports found
            Log.d("getReportsFromDB", "No Messages");
            String[] openTicketsArray = new String[]{"No Reports Available"};
            reports.setAdapter(new ArrayAdapter<String>(this,
                    android.R.layout.simple_list_item_1, openTicketsArray));
        }
    }

    private void getReport(String sReportNumber) {
        Intent iCompleteReport = new Intent(getApplicationContext(), DVIRCompleteReportActivity.class);
        iCompleteReport.putExtra("DVIRNo", sReportNumber);
        startActivity(iCompleteReport);
    }

    //This is when the initial DVIR was only partialy completed
    private void completeReport() {
        Intent iCompleteReport = new Intent(getApplicationContext(), DVIREntryActivity.class);
        startActivity(iCompleteReport);
    }
}
