package com.perfectech.tis.activities;

import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Spinner;
import android.widget.TextView;

import com.perfectech.tis.DBLogs;
import com.perfectech.tis.DBParameters;
import com.perfectech.tis.R;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

public class SettingsActivity extends AppCompatActivity {
    private String DateFormats[] = {
            "MM/dd/yyyy HH:mm",
            "MM/dd/yyyy hh:mm aa",
            "dd/MM/yyyy HH:mm",
            "dd/MM/yyyy hh:mm aa"
    };
    private static final String sDBName = "TIS.db";
    static int iVersionVal = 5;
    Spinner spDateFormat = null;
    Spinner spDebug = null;
    private static String sUserID = "";
    String sCurrentVersion;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_settings);
        SharedPreferences prefs = getSharedPreferences("TISSharedPrefs",
                Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = prefs.edit();
        String sLoggedInUserCompany = prefs.getString("LoggedInUserCompany", "No Value");
        sUserID = prefs.getString("UserID", "No Value");
        sCurrentVersion = prefs.getString("TISVersion", "No Value");
        TextView otvUserName = (TextView) findViewById(R.id.tvUserName);
        otvUserName.setText(sLoggedInUserCompany.substring(0,sLoggedInUserCompany.indexOf('-')));
        spDateFormat = (Spinner) findViewById(R.id.spSettingDate);
        spDateFormat.setOnItemSelectedListener(
                new AdapterView.OnItemSelectedListener() {
                    @Override
                    public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                        updateDateFormat(position);
                    }

                    @Override
                    public void onNothingSelected(AdapterView<?> parent) {

                    }
                }
        );
        spDebug = (Spinner) findViewById(R.id.spDebug);
        spDebug.setOnItemSelectedListener(
                new AdapterView.OnItemSelectedListener() {
                    @Override
                    public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                        updateDebug(position);
                    }

                    @Override
                    public void onNothingSelected(AdapterView<?> parent) {

                    }
                }
        );
        populateDateFormatSpinner();
        populateDebugSpinner();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        //getMenuInflater().inflate(R.menu.menu_settings, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_settings) {
            return true;
        }
        if (id == R.id.action_icon_meanings) {
            Intent i = new Intent(getApplicationContext(), HelpActivity.class);
            i.putExtra("ScreenID", "IconMeanings");
            startActivity(i);
            return true;
        }

        return super.onOptionsItemSelected(item);
    }

    private void populateDateFormatSpinner(){
        Date dDateNow = new Date();
        List<String> arraySpinner = new ArrayList<String>();
        ArrayAdapter<String> adapter;
        for (int i=0; i < DateFormats.length; i++) {
            SimpleDateFormat sdfLocal = new SimpleDateFormat(DateFormats[i]);
            String strDate = sdfLocal.format(dDateNow);
            arraySpinner.add(strDate);
        }
        adapter = new ArrayAdapter<String>(getApplicationContext(),
                R.layout.setting_date_view, arraySpinner);
        adapter.setDropDownViewResource(R.layout.setting_date_spinner);
        spDateFormat.setAdapter(adapter);
        try {
            //Find the current date format selection in the parameters table
            DBParameters oDBParameters = new DBParameters();
            String sDateFormat = oDBParameters.readParameter(
                    "DateFormat", getApplicationContext(), sDBName,
                    iVersionVal);
            spDateFormat.setSelection(Integer.parseInt(sDateFormat));
        } catch(Exception ex) {
            //To do
        }
    }

    private void populateDebugSpinner() {
        List<String> arraySpinner = new ArrayList<String>();
        ArrayAdapter<String> adapter;
        arraySpinner.add("No");
        arraySpinner.add("Yes");
        adapter = new ArrayAdapter<String>(getApplicationContext(),
                R.layout.setting_date_view,arraySpinner);
        adapter.setDropDownViewResource(R.layout.setting_date_spinner);
        spDebug.setAdapter(adapter);
        try {
            //Find the current date format selection in the parameters table
            DBParameters oDBParameters = new DBParameters();
            String sDebugFormat = oDBParameters.readParameter(
                    "DebugMode", getApplicationContext(), sDBName,
                    iVersionVal);
            spDebug.setSelection(Integer.parseInt(sDebugFormat));
        } catch(Exception ex) {
            //To do
        }
    }

    private void updateDateFormat(int iformatIndex) {
        DBParameters oDBParameters = new DBParameters();
        try {
            oDBParameters.updateParameter("DateFormat",
                    String.valueOf(iformatIndex), getApplicationContext(), sDBName,
                    iVersionVal);
        } catch(Exception ex) {
            // To do
        }
    }

    private void updateDebug(int idebugIndex) {
        DBParameters oDBParameters = new DBParameters();
        try {
            oDBParameters.updateParameter("DebugMode",
                    String.valueOf(idebugIndex), getApplicationContext(), sDBName,
                    iVersionVal);

            if (idebugIndex == 1) {
                DBLogs.insert("Device Manufacturer: " + android.os.Build.MANUFACTURER, "SettingsActivity",
                        getApplicationContext(), sDBName, iVersionVal);
                DBLogs.insert("Device Model: " + android.os.Build.MODEL, "SettingsActivity",
                        getApplicationContext(), sDBName, iVersionVal);
                DBLogs.insert("Device Serial #: " + android.os.Build.SERIAL, "SettingsActivity",
                        getApplicationContext(), sDBName, iVersionVal);
                DBLogs.insert("Device OS Version #: " + android.os.Build.VERSION.RELEASE, "SettingsActivity",
                        getApplicationContext(), sDBName, iVersionVal);
                DBLogs.insert("Software Version #: " + sCurrentVersion, "SettingsActivity",
                        getApplicationContext(), sDBName, iVersionVal);
            }
        } catch(Exception ex) {
            // To do
        }
    }
}
