package com.perfectech.tis.activities;

import android.content.ContentValues;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.ImageView;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.provider.MediaStore;
import android.net.Uri;
import android.os.Environment;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.database.Cursor;

import com.perfectech.tis.DateConvertClass;
import com.perfectech.tis.R;
import com.perfectech.tis.syncClasses.SyncTasks;
import com.perfectech.tis.TISDBHelper;
import com.perfectech.tis.util.BitmapScaler;

import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStream;


public class UserProfile extends AppCompatActivity {

    private static final String sDBName = "TIS.db";
    private static final int iVersionVal = 5;
    private static String sAndroidID = "";
    private static String sUserID = "";
    private static ArrayList<ArrayList<String>> userArray;
    private String sImageName = "";
    private String sSigName = "";
    private String sPrivateSigDir = "signatures";
    private String sPrivateImageDir = "images";
    private ImageView mImageView;
    private ImageView mSigImageView;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_user_profile);

        SharedPreferences prefs = getSharedPreferences("TISSharedPrefs",
                Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = prefs.edit();
        String sLoggedInUserCompany = prefs.getString("LoggedInUserCompany", "No Value");
        sUserID = prefs.getString("UserID","No Value");
        sUserID = sUserID.trim();

        mImageView = (ImageView) findViewById(R.id.ivUser);
        mImageView.setOnClickListener(new View.OnClickListener() {

            @Override
            public void onClick(View view) {
                // do stuff
                selectImage();
            }

        });

        // get user data for the current logged in user
        TISDBHelper oDB = new TISDBHelper(getApplicationContext(), sDBName,
                null, iVersionVal);
        try {
            oDB.createDataBase();
        } catch (IOException ioe) {
            throw new Error("Unable to create database");
        }

        oDB.openDataBase();
        String sSQL = "Select FirstName, LastName, EmailAddress, DriverImageTaken, CompanyName ";
        sSQL += "from EmployeeMaster ";
        sSQL += " where lower(UserID) = lower('" + sUserID + "')";

        Log.d("sSQL", sSQL);

        userArray = oDB.selectRecordsFromDBList(sSQL, null);

        // closes the database
        try {
            oDB.close();
        } catch (Throwable e) {
            e.printStackTrace();
        }

        String sFirstName = "";
        String sLastName = "";
        String sEmailAddress = "";
        String sCompanyName = "";

        if (userArray.size() > 0) {
            for (ArrayList<String> sValue : userArray) {
                sFirstName  = sValue.get(0);
                sLastName = sValue.get(1);
                sEmailAddress = sValue.get(2);
                sImageName = sValue.get(3);
                sCompanyName = sValue.get(4);
            }
        }

        // fill in the field on the form
        EditText etFirstName = (EditText) findViewById(R.id.etFirstName);
        etFirstName.setText(sFirstName);
        EditText etLastName = (EditText) findViewById(R.id.etLastName);
        etLastName.setText(sLastName);
        EditText etEmailAddress = (EditText) findViewById(R.id.etEmailAddress);
        etEmailAddress.setText(sEmailAddress);
        EditText etCompanyName = (EditText) findViewById(R.id.etCompanyName);
        etCompanyName.setText(sCompanyName);
        //Display the user image
        displayImage();

        sSigName = "sig_" + sUserID + ".png";
        mSigImageView = (ImageView) findViewById(R.id.ivDriverSig);
        mSigImageView.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                Intent i = new Intent(getApplicationContext(), SigCapture.class);
                i.putExtra("InType","USER");
                startActivity(i);
            }
        });

        displaySigImage();

    }

    public void onResume()
    {
        super.onResume();
        sSigName = "sig_" + sUserID + ".png";
        mSigImageView = (ImageView) findViewById(R.id.ivDriverSig);
        displaySigImage();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_user_profile, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_save) {
            saveData(1);
            return true;
        }
        if (id == R.id.action_cancel) {
            finish();
            return true;
        }
        if (id == R.id.action_signature) {
            Intent i = new Intent(getApplicationContext(), SigCapture.class);
            i.putExtra("InType","USER");
            startActivity(i);
            return true;
        }
        if (id == R.id.action_help) {
            Intent i = new Intent(getApplicationContext(), HelpActivity.class);
            i.putExtra("ScreenID","UserProfile");
            startActivity(i);
            return true;
        }
        if (id == R.id.action_about) {
            Intent i = new Intent(getApplicationContext(), AboutActivity.class);
            startActivity(i);
            return true;
        }

        return super.onOptionsItemSelected(item);
    }

    private void saveData(int iExit) {
        String sFirstName = "";
        String sLastName = "";
        String sEmailAddress = "";
        String sPassword = "";
        String sConfirmPassword = "";
        String sCompanyName = "";

        // get information from the fields
        EditText etFirstName = (EditText) findViewById(R.id.etFirstName);
        EditText etLastName = (EditText) findViewById(R.id.etLastName);
        EditText etEmailAddress = (EditText) findViewById(R.id.etEmailAddress);
        EditText etPassword = (EditText) findViewById(R.id.etPassword);
        EditText etConfirmPassword = (EditText) findViewById(R.id.etConfirmPassword);
        EditText etCompanyName = (EditText) findViewById(R.id.etCompanyName);
        TextView tvError = (TextView) findViewById(R.id.tvError);

        sFirstName = etFirstName.getText().toString();
        sLastName = etLastName.getText().toString();
        sEmailAddress = etEmailAddress.getText().toString();
        sPassword = etPassword.getText().toString();
        sConfirmPassword = etConfirmPassword.getText().toString();
        sCompanyName = etCompanyName.getText().toString();
        //Make sure to update the images just in case they changed
        displayImage();
        displaySigImage();

        // need to confirm that the passwords match only if they are doing a password change
        if (!sPassword.equals(sConfirmPassword) && sPassword.length() > 0) {
            tvError.setVisibility(View.VISIBLE);
        } else {
            try {
                TISDBHelper oDB = new TISDBHelper(getApplicationContext(),
                        sDBName, null, iVersionVal);
                oDB.createDataBase();

                oDB.openDataBase();
                ContentValues oValues = new ContentValues();
                oValues.put("FirstName", sFirstName);
                oValues.put("LastName", sLastName);
                oValues.put("EmailAddress", sEmailAddress);
                oValues.put("DriverImageTaken",sImageName);
                oValues.put("CompanyName", sCompanyName);
                if (sPassword.length() > 0) {
                    oValues.put("Password", sPassword);
                    //Make sure to update the remember me password
                    SharedPreferences tisLoginPrefs = getSharedPreferences("TISlogin",
                            Context.MODE_PRIVATE);
                    SharedPreferences.Editor editor = tisLoginPrefs.edit();
                    editor.putString("password", sPassword.toString());
                    editor.apply();
                }

                oDB.updateRecordsInDB("EmployeeMaster", "UserID = '"
                        + sUserID.toLowerCase() + "'", oValues);
                oDB.close();

                queueTheUpdates(iExit);
            } catch (Exception ex) {
                Log.d("Error:", ex.toString());
            }
            if (iExit == 1) {
                finish();
            }
        }
    }

    private void queueTheUpdates(int iUpdateType) {
        // write the message to the Messages table
        Calendar c = Calendar.getInstance();
        DateConvertClass oDateConvertClass = new DateConvertClass();
        oDateConvertClass.oContext = getApplicationContext();
        String sCurrentDateTime = oDateConvertClass.GMTTimestamp();

        if (iUpdateType == 1) {
            // This is a general profile update that is not a picture update
            SyncTasks syncTasks = new SyncTasks();
            syncTasks.oContext = getApplicationContext();
            syncTasks.sRecType = "UP";
            syncTasks.sRecID = sUserID;
            syncTasks.sDateTime = sCurrentDateTime;
            syncTasks.sQueued = "0";
            syncTasks.sDateTimeQueued = "";
            syncTasks.sCompleted = "0";
            syncTasks.sDateTimeCompleted = "";
            syncTasks.write();
        }
        if (iUpdateType == 0) {
            // This is a picture update
            SyncTasks syncTasks = new SyncTasks();
            syncTasks.oContext = getApplicationContext();
            syncTasks.sRecType = "UPhoto";
            syncTasks.sRecID = sImageName;
            syncTasks.sDateTime = sCurrentDateTime;
            syncTasks.sQueued = "0";
            syncTasks.sDateTimeQueued = "";
            syncTasks.sCompleted = "0";
            syncTasks.sDateTimeCompleted = "";
            syncTasks.write();
        }

    }

    private void selectImage() {

        final CharSequence[] options = { "Take Photo", "Choose from Gallery","Cancel" };

        AlertDialog.Builder builder = new AlertDialog.Builder(UserProfile.this);
        builder.setTitle("Add Photo!");
        builder.setItems(options, new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int item) {
                if (options[item].equals("Take Photo"))
                {
                    Intent intent = new Intent(MediaStore.ACTION_IMAGE_CAPTURE);
                    File storageDir = new File(Environment.getExternalStorageDirectory() + "/tis");
                    File storageDirTemp = new File(storageDir, "temp");
                    if (makeAndClearPictureDirectorys(storageDir)) {
                        //String sPath = getApplicationContext().getFilesDir().toString();
                        sImageName = sUserID.replace('@','_').replace('.','_') + "_image.jpg";
                        File f = new File(storageDirTemp, sImageName);
                        intent.putExtra(MediaStore.EXTRA_OUTPUT, Uri.fromFile(f));
                        startActivityForResult(intent, 1);
                    }
                }
                else if (options[item].equals("Choose from Gallery"))
                {
                    Intent intent = new Intent(Intent.ACTION_PICK,android.provider.MediaStore.Images.Media.EXTERNAL_CONTENT_URI);
                    startActivityForResult(intent, 2);

                }
                else if (options[item].equals("Cancel")) {
                    dialog.dismiss();
                }
            }
        });
        builder.show();
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        ImageView mImageView = (ImageView) findViewById(R.id.ivUser);
        super.onActivityResult(requestCode, resultCode, data);
        if (resultCode == RESULT_OK) {
            //Place the picture in a private directory images
            File destinationDir = getDir(sPrivateImageDir, Context.MODE_PRIVATE);
            if (requestCode == 1) {
                // Verify the directory we want to store things in exists and if not then create it
                File storageDir = new File(Environment.getExternalStorageDirectory() + "/tis");
                File storageDirTemp = new File(storageDir, "temp");
                try {
                    sImageName = sUserID.replace('@', '_').replace('.', '_') + "_image.jpg";
                    File origFile = new File(storageDirTemp, sImageName);
                    BitmapScaler scaler = new BitmapScaler(origFile, 400);
                    Bitmap bmp = Bitmap.createBitmap(scaler.getScaled());
                    OutputStream stream = new FileOutputStream(destinationDir.toString() + "/" + sImageName);
                    bmp.compress(Bitmap.CompressFormat.JPEG, 90, stream);
                    //Delete the photo that was taken
                    origFile.delete();
                    saveData(0);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            } else if (requestCode == 2) {

                Uri selectedImage = data.getData();
                String[] filePath = {MediaStore.Images.Media.DATA};
                Cursor c = getContentResolver().query(selectedImage, filePath, null, null, null);
                c.moveToFirst();
                int columnIndex = c.getColumnIndex(filePath[0]);
                String picturePath = c.getString(columnIndex);
                c.close();
                try {
                    sImageName = sUserID.replace('@', '_').replace('.', '_') + "_image.jpg";
                    File origFile = new File(picturePath);
                    BitmapScaler scaler = new BitmapScaler(origFile, 400);
                    Bitmap bmp = Bitmap.createBitmap(scaler.getScaled());
                    OutputStream stream = new FileOutputStream(destinationDir.toString() + "/" + sImageName);
                    bmp.compress(Bitmap.CompressFormat.JPEG, 90, stream);
                    saveData(0);
                } catch (Exception ex) {
                    ex.printStackTrace();
                }
            }
        }
    }

    private boolean makeAndClearPictureDirectorys(File fStorageDir) {
        File fStorageDirTemp = new File(fStorageDir, "temp");
        if (!fStorageDirTemp.exists()) {
            //mkdirs makes the directory and parent directory if it does not exist
            fStorageDirTemp.mkdirs();
        }
        //Now see if there is anything in the temp dir and delete.  There should never be anythin
        //in the temp directory since it will hold the full size file until it is shrunk
        if (fStorageDirTemp.isDirectory()) {
            File[] files = fStorageDirTemp.listFiles();
            for (File file : files)
            {
                if (!file.delete())
                {
                    System.out.println("Failed to delete " + file);
                }
            }
        }
        return fStorageDirTemp.isDirectory();
    }

    private void displayImage() {
        try {
            File storageDir = getDir(sPrivateImageDir, Context.MODE_PRIVATE);
            File f = new File(storageDir, sImageName);
            FileInputStream in = new FileInputStream(f);

            BitmapFactory.Options bitmapOptions = new BitmapFactory.Options();
            bitmapOptions.inSampleSize = 2;
            Bitmap bitmap = BitmapFactory.decodeStream(in, null, bitmapOptions);

            mImageView.setImageBitmap(bitmap);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    private void displaySigImage() {
        try {
            File storageDir = getDir(sPrivateSigDir, Context.MODE_PRIVATE);
            File f = new File(storageDir, sSigName);
            FileInputStream in = new FileInputStream(f);

            BitmapFactory.Options bitmapOptions = new BitmapFactory.Options();
            bitmapOptions.inSampleSize = 4;
            Bitmap bitmap = BitmapFactory.decodeStream(in, null, bitmapOptions);

            mSigImageView.setImageBitmap(bitmap);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
}
