/********************************************************************************/
/* Class:	dataSyncService.java                                                */
/* Date: 	May 1, 2012                                                         */
/* Version:	1.0.0                                                               */
/* Author:	Joe A Cairns                                                        */
/* Description                                                                  */
/*    This is the main class file for handling sync to server                   */
/*                                                                              */
/* Modification Log                                                             */
/*                                                                              */
/*           Copyright (c) 2012 PerfecTech                                      */
/********************************************************************************/
package com.perfectech.tis.syncClasses;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Timer;
import java.util.TimerTask;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.res.Resources;
import android.os.IBinder;
import android.provider.Settings;
import android.util.Log;
import android.content.SharedPreferences.Editor;

import com.perfectech.tis.DBLogs;
import com.perfectech.tis.DBParameters;
import com.perfectech.tis.DateConvertClass;
import com.perfectech.tis.JobClass;;
import com.perfectech.tis.TISDBHelper;
import com.perfectech.tis.activities.LongOpenJob;
import com.perfectech.tis.objects.Jobs;

/**
 * @author jcairns
 * 
 */
public class dataSyncService extends Service {
	/********************************************************************************/
	/* Define Global Variables */
	/********************************************************************************/
	private Timer timer = new Timer();
	private static long UPDATE_INTERVAL = 60000; // every 1 mins the sync will check
	private static final String sDBName = "TIS.db";
	private long lLongJob = 240; //240 minutes or 4 hours for log open job
	static int iVersionVal = 5;
	private String sDeviceID;
	private String sAndroidID;
	//private String sSyncType = "N";
	//private static String sSoftwareUpdateURL = "";
	private static String sSoftwareUpdateVersionCheck = "";
	private static String sSoftwareName = "";
	//private static String sNextVersion = "";
	private static String sCurrentVersion = "";
	private static Resources res;
	private static boolean debug_mode = false;
	private static Date oLastReminderCheck = null;

	@Override
	public IBinder onBind(Intent arg0) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public void onCreate() {
		super.onCreate();
        if (debug_mode == true) {
            DBLogs.insert("Starting up background sync service", "dataSyncService",
                    getApplicationContext(), sDBName, iVersionVal);
        }

		_startService();
	}

	@Override
	public void onDestroy() {
		super.onDestroy();
		_shutdownService();
	}

	/********************************************************************************/
	/* Define Global Variables */
	/********************************************************************************/
	private void _startService() {
		// need to get the number of minutes for the sync delay
		DBParameters oDBParameters = new DBParameters();

		String sSyncDelayMins = "";

		try {
			sSyncDelayMins = oDBParameters.readParameter("SYNC_DELAY_MINS",
					getApplicationContext(), sDBName, iVersionVal);
		} catch (Exception e) {

		}

		if (sSyncDelayMins.length() < 1)
			sSyncDelayMins = "5";

		UPDATE_INTERVAL = ((Long.parseLong(sSyncDelayMins) * 60) * 1000);

		timer.scheduleAtFixedRate(new TimerTask() {
			public void run() {
				DateConvertClass oDateConvertClass = new DateConvertClass();
				oDateConvertClass.oContext = getApplicationContext();
				String sCurrentDateTime = oDateConvertClass.GMTTimestamp();

				DBParameters oDBParameters = new DBParameters();
				try {
					String sDebugMode = oDBParameters.readParameter("DebugMode",
							getApplicationContext(), sDBName, iVersionVal);
					if (sDebugMode.equals("1")) {
						debug_mode = true;
					} else {
						debug_mode = false;
					}
				} catch(Exception ex) {
					// To do
				}
                if (debug_mode == true) {
                    DBLogs.insert("SYNC_DELAY_MINS: " + String.valueOf((UPDATE_INTERVAL / 1000) / 60) + " Minutes",
                            "dataSyncService:_startService()", getApplicationContext(),
                            sDBName, iVersionVal);
                }

				if (debug_mode == true) {
					DBLogs.insert("Start dataSyncService",
							"dataSyncService:_startService().run()",
							getApplicationContext(), sDBName, iVersionVal);
				}
				if (TISCommon.isOnline()) {
					if (debug_mode == true) {
						DBLogs.insert("We are online need to do a sync",
								"dataSyncService._startService().isOnline",
								getApplicationContext(), sDBName, iVersionVal);
					}

					// if the system is connected to the web then
					// let's see if we need to sync
					if (checkIfSyncNeeded()) {

						// need to loop through all the sync tasks
						int iMaxRecs = SyncTasks.getNumRecs();

						for (int iCounter = 0; iCounter < iMaxRecs; iCounter++) {
							if (debug_mode == true) {
								DBLogs.insert("Have data to be sync'd",
										"dataSyncService.checkIfSyncNeeded",
										getApplicationContext(), sDBName,
										iVersionVal);
							}
							// get the next record to process
							SyncTasks.getNexRec();
							SyncTasks.read(SyncTasks.sID);
							Intent SyncIntent = new Intent(getApplicationContext(), SyncIntentService.class);
							SyncIntent.putExtra("RecType", SyncTasks.sRecType);
							SyncIntent.putExtra("ID", SyncTasks.sID);
							SyncIntent.putExtra("RecID", SyncTasks.sRecID);
							startService(SyncIntent);
							//Mark the record as queued.
							SyncTasks.updatequeued("1", sCurrentDateTime, SyncTasks.sID);
						}
					} else {
                        if (debug_mode == true) {
                            DBLogs.insert("There is NOT any data to be sync'd",
                                    "dataSyncService.checkIfSyncNeeded",
                                    getApplicationContext(), sDBName,
                                    iVersionVal);
                        }
                    }

					Intent LogsIntent = new Intent(getApplicationContext(), SyncIntentService.class);
					LogsIntent.putExtra("RecType", "LOGS");
					LogsIntent.putExtra("ID", "NOTHING");
					LogsIntent.putExtra("RecID", "NOTHING");
					startService(LogsIntent);
					Intent MsgIntent = new Intent(getApplicationContext(), SyncIntentService.class);
					MsgIntent.putExtra("RecType", "MESSAGE");
					MsgIntent.putExtra("ID", "NOTHING");
					MsgIntent.putExtra("RecID", "NOTHING");
					startService(MsgIntent);
					Intent JobsIntent = new Intent(getApplicationContext(), SyncIntentService.class);
					JobsIntent.putExtra("RecType", "JOBS");
					JobsIntent.putExtra("ID", "NOTHING");
					JobsIntent.putExtra("RecID", "NOTHING");
					startService(JobsIntent);
					checkOpenJobLength();
				} else {
                    if (debug_mode == true) {
                        DBLogs.insert("We are not online. Will not sync at this time.",
                                "dataSyncService._startService.isOnline",
                                getApplicationContext(), sDBName, iVersionVal);
                    }
                }
			}
		}, 0, UPDATE_INTERVAL);
	}

	private void _shutdownService() {
		if (timer != null)
			timer.cancel();
	}

	private boolean checkIfSyncNeeded() {
		boolean bRtnVal = false;

		// let's see if there any rows in the table to process
		SyncTasks.oContext = getApplicationContext();
		if (SyncTasks.getNumRecs() > 0) {
			bRtnVal = true;
		} else {
			bRtnVal = false;
		}

		return bRtnVal;
	}

	/********************************************************************************/
	/* This method will get the user id */
	/********************************************************************************/
	private String getUserID(String sUserID) {
		String sDriverNo = "";

		try {
			TISDBHelper oDB = new TISDBHelper(getApplicationContext(), sDBName,
					null, iVersionVal);
			try {
				oDB.createDataBase();
			} catch (IOException ioe) {
				DBLogs.insert("Error: " + ioe.toString(),
						"dataSyncServie.getUserID", getApplicationContext(),
						sDBName, iVersionVal);
				throw new Error("Unable to create database");
			}

			oDB.openDataBase();

			String sSQL = "select DriverNo from EmployeeMaster where lower(UserID) = '";
			sSQL += sUserID.toLowerCase() + "'";

			ArrayList<String> sStringList = oDB.getList(sSQL, null);
			oDB.close();

			String[] userInfoArray = new String[sStringList.size()];
			userInfoArray = sStringList.toArray(userInfoArray);

			sDriverNo = userInfoArray[0];

		} catch (Exception ex) {
			DBLogs.insert("Error: " + ex.toString(),
					"dataSyncServie.getUserID", getApplicationContext(),
					sDBName, iVersionVal);
			Log.d("dataSyncServie.getUserID(): ", ex.toString());
		}

		return sDriverNo;
	}

	/*****************************************************************************************/
	/* This method will get the assigned device ID */
	/*****************************************************************************************/
	private void checkDevice() {
		try {
			TISDBHelper oDB = new TISDBHelper(getApplicationContext(), sDBName,
					null, iVersionVal);
			try {
				oDB.createDataBase();
			} catch (IOException ioe) {
				DBLogs.insert("Error: " + ioe.toString(),
						"dataSyncService.checkDevice:createDB",
						getApplicationContext(), sDBName, iVersionVal);
				throw new Error("Unable to create database");
			}

			oDB.openDataBase();

			sAndroidID = Settings.System.getString(getContentResolver(),
					Settings.System.ANDROID_ID);

			// select the ticket information from the DB
			String sSQL = "select DeviceID ";
			sSQL += "from DeviceMaster ";
			sSQL += "where AndroidID =  '" + sAndroidID + "' ";

			ArrayList<String> deviceArray = oDB.getList(sSQL, null);

			// closes the database
			try {
				oDB.close();
			} catch (Throwable e) {
				DBLogs.insert("Error: " + e.toString(),
						"dataSyncService.checkDevice:closeDB",
						getApplicationContext(), sDBName, iVersionVal);
				e.printStackTrace();
			}

			String[] deviceInfoArray;

			deviceInfoArray = new String[deviceArray.size()];
			deviceInfoArray = deviceArray.toArray(deviceInfoArray);

			if (deviceArray.size() > 0) {
				sDeviceID = deviceInfoArray[0];
			} else {
			}
		} catch (Exception e) {
			DBLogs.insert("Error: " + e.toString(),
					"dataSyncService.checkDevice", getApplicationContext(),
					sDBName, iVersionVal);
		}
	}

	private void checkOpenJobLength() {
		JobClass oOpenJobs = new JobClass();
		oOpenJobs.oContext = getApplicationContext();
		Jobs oJob = oOpenJobs.readWithPaused();
		if (oJob.job_num != null) {
			DateConvertClass oDateConvertClass = new DateConvertClass();
			oDateConvertClass.oContext = getApplicationContext();
			String sGMTNow = oDateConvertClass.GMTTimestamp();
			SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
			SharedPreferences activeJobs = getApplicationContext().getSharedPreferences("activeJobs", Context.MODE_PRIVATE);
			SharedPreferences lastReminderCheck = getApplicationContext().getSharedPreferences("lastReminderCheck", Context.MODE_PRIVATE);
			Editor editor = lastReminderCheck.edit();
			//String sGMTStart = activeJobs.getString(oJob.job_num, "no value");
			//if (!sGMTStart.equals("no value")) {
				long lDiffMin = 0;
				long lDiffHours = 0;
				long iDiffInDays = 0;
				try {
					Date dDateStart = format.parse(oJob.date_time);
					Date dDateNow = format.parse(sGMTNow);

					String sLastReminderCheck = "";
					sLastReminderCheck = lastReminderCheck.getString("LastReminderCheck", null);
					if (sLastReminderCheck == null) {
						sLastReminderCheck = sGMTNow;
						editor.putString("LastReminderCheck",sGMTNow);
						editor.commit();
					}
					oLastReminderCheck = format.parse(sLastReminderCheck);

					long lDiff = dDateNow.getTime() - oLastReminderCheck.getTime();
					lDiffMin = lDiff / (60 * 1000) % 60;
					lDiffHours = lDiff / (60 * 60 * 1000);
					iDiffInDays = (int) lDiff / (1000 * 60 * 60 * 24);

					if (lDiffHours > 0) {
						lDiffHours = lDiffHours * 60;
					}

					if (iDiffInDays > 0) {
						iDiffInDays = iDiffInDays * 24 * 60;
					}

 					if ((lDiffMin + lDiffHours + iDiffInDays) >= lLongJob) {
						Intent iTest = new Intent(getApplicationContext(), LongOpenJob.class);
						iTest.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
						iTest.addFlags(Intent.FLAG_ACTIVITY_SINGLE_TOP);
						iTest.putExtra("jobNumber", oJob.job_num);
						iTest.putExtra("jobDestinationCity", oJob.delivery_city);
						iTest.putExtra("jobID", oJob.id);
						iTest.putExtra("jobStatus", oJob.status_name);
						startActivity(iTest);
						editor.putString("LastReminderCheck",sGMTNow);
						editor.commit();
					}
				} catch (Exception ex) {
					DBLogs.insert("Date Exception: " + ex.toString(),
							"dataSyncServices.checkOpenJobLength",
							getApplicationContext(), sDBName, iVersionVal);
				}

			//}
		}
	}

}
